@@ -158,7 +158,7 @@ let rec selectAction batch actions tryagain =
   | Some i -> i)
 
 let alwaysDisplayDetails ri =
-  alwaysDisplay ((Uicommon.details2string ri "\n  ") ^ "\n")
+  alwaysDisplay ((Uicommon.details2string ri "  ") ^ "\n")
 
 let displayDetails ri =
   if not (Prefs.read silent) then alwaysDisplayDetails ri
@@ -189,7 +189,7 @@ let interact rilist =
   display ("\n" ^ Uicommon.roots2string() ^ "\n");
   let rec loop prev =
     function
-      [] -> (ConfirmBeforeProceeding, List.rev prev)
+      [] -> (ConfirmBeforeProceeding, Safelist.rev prev)
     | ri::rest as ril ->
         let next() = loop (ri::prev) rest in
         let repeat() = loop prev ril in
@@ -266,7 +266,15 @@ let interact rilist =
                      newLine ();
                      Uicommon.showDiffs ri
                        (fun title text ->
-                          Printf.printf "\n%s\n\n%s\n\n" title text)
+                          try
+                            let pager = Sys.getenv "PAGER" in
+                            restoreTerminal ();
+                            let out = Unix.open_process_out pager in
+                            Printf.fprintf out "\n%s\n\n%s\n\n" title text;
+                            let _ = Unix.close_process_out out in
+                            setupTerminal ()
+                          with Not_found ->
+                            Printf.printf "\n%s\n\n%s\n\n" title text)
                        (fun s -> Printf.printf "%s\n" s)
                        Uutil.File.dummy;
                      repeat()));
@@ -277,7 +285,8 @@ let interact rilist =
                   ("list all suggested changes"),
                   (fun () -> display "\n";
                      Safelist.iter
-                       (fun ri -> displayri ri; alwaysDisplayDetails ri)
+                       (fun ri -> displayri ri; display "\n  ";
+                                  alwaysDisplayDetails ri)
                        ril;
                      display "\n";
                      repeat()));
@@ -291,7 +300,7 @@ let interact rilist =
                  (["g"],
                   ("proceed immediately to propagating changes"),
                   (fun() ->
-                     (ProceedImmediately, List.rev_append prev ril)));
+                     (ProceedImmediately, Safelist.rev_append prev ril)));
                  (["q"],
                   ("exit " ^ Uutil.myName ^ " without propagating any changes"),
                   fun () -> raise Sys.Break);
@@ -334,12 +343,12 @@ let verifyMerge title text =
         (fun () -> false));
        ]
       (fun () -> display "Commit results of merge? ")
-  end 
+  end
 
 let doTransport reconItemList =
   let totalBytesToTransfer =
     ref
-      (List.fold_left
+      (Safelist.fold_left
          (fun l ri -> Uutil.Filesize.add l (Common.riLength ri))
          Uutil.Filesize.zero reconItemList) in
   let totalBytesTransferred = ref Uutil.Filesize.zero in
@@ -364,7 +373,7 @@ let doTransport reconItemList =
   if not (Prefs.read Trace.terse) && (Prefs.read Trace.debugmods = []) then
     Uutil.setProgressPrinter showProgress;
 
-  Transport.logStartTime();
+  Transport.start ();
   let fFailedPaths = ref [] in
   let uiWrapper ri f =
     catch f
@@ -384,7 +393,8 @@ let doTransport reconItemList =
     | ri :: rest when pRiThisRound ri ->
         loop rest
           (uiWrapper ri
-             (fun () -> Transport.transportItem ri (Uutil.File.ofLine 0) verifyMerge)
+             (fun () -> Transport.transportItem ri
+                          (Uutil.File.ofLine 0) verifyMerge)
            :: actions)
           pRiThisRound
     | _ :: rest ->
@@ -397,7 +407,7 @@ let doTransport reconItemList =
   Lwt_unix.run
     (let actions = loop reconItemList [] Common.isDeletion in
     Lwt_util.join actions);
-  Transport.logEndTime();
+  Transport.finish ();
 
   Uutil.setProgressPrinter (fun _ _ _ -> ());
   Util.set_infos "";
@@ -410,7 +420,7 @@ let setWarnPrinterForInitialization()=
             alwaysDisplay "Error: ";
             alwaysDisplay s;
             alwaysDisplay "\n";
-            exit 1)
+            exit Uicommon.fatalExit)
 
 let setWarnPrinter() =
   Util.warnPrinter :=
@@ -426,8 +436,10 @@ let setWarnPrinter() =
                  (["n";"q";"x"],
                     ("Exit"),
                     fun()->
+                      alwaysDisplay "\n";
+                      restoreTerminal ();
                       Lwt_unix.run (Update.unlockArchives ());
-                      exit 1)]
+                      exit Uicommon.fatalExit)]
                 (fun()-> display  "Press return to continue.")
             end)
 
@@ -508,9 +520,32 @@ let rec interactAndPropagateChanges reconItemList
       (fun () -> display "Proceed with propagating updates? ")
   end
 
+let checkForDangerousPath dangerousPaths =
+  if dangerousPaths <> [] then begin
+    alwaysDisplayAndLog (Uicommon.dangerousPathMsg dangerousPaths);
+    if Prefs.read Globals.batch then begin
+      (* FIX: there should probably be an option to force proceeding *)
+      alwaysDisplay "Aborting...\n"; restoreTerminal ();
+      exit Uicommon.fatalExit
+    end else begin
+      displayWhenInteractive "Do you really want to proceed? ";
+      selectAction
+        None
+        [(["y"],
+          "Continue",
+          (fun() -> ()));
+         (["n"; "q"; "x"; ""],
+          "Exit",
+          (fun () -> alwaysDisplay "\n"; restoreTerminal ();
+                     exit Uicommon.fatalExit))]
+        (fun () -> display "Do you really want to proceed? ")
+    end
+  end
+
 let synchronizeOnce() =
   Trace.status "Looking for changes";
-  let (reconItemList, anyEqualUpdates) = Recon.reconcileAll (Update.findUpdates()) in
+  let (reconItemList, anyEqualUpdates, dangerousPaths) =
+    Recon.reconcileAll (Update.findUpdates()) in
   if reconItemList = [] then begin
     (if anyEqualUpdates then
       Trace.status ("Nothing to do: replicas have been changed only "
@@ -519,11 +554,12 @@ let synchronizeOnce() =
        Trace.status "Nothing to do: replicas have not changed since last sync.");
     (Uicommon.perfectExit, [])
   end else begin
+    checkForDangerousPath dangerousPaths;
     let (anySkipped, anyFailures, failedPaths) =
       interactAndPropagateChanges reconItemList in
     let exitStatus = Uicommon.exitCode(anySkipped,anyFailures) in
     (exitStatus, failedPaths)
-  end 
+  end
 
 let synchronizeUntilNoFailures () =
   let initValueOfPathsPreference = Prefs.read Globals.paths in
